#ifndef __niflexrioapi_h__
#define __niflexrioapi_h__

#include <niflexrioapi_defines.h>
#include <niflexrioapi_enums.h>
#include <niflexrioapi_attributes.h>
#include <niflexrioapi_wfminfo.h>
#include <IviVisaType.h>

#define NIFLEXRIO_MAX_ERROR_SIZE (4096)

typedef int32_t FlexRIO_Status;
typedef uint32_t FlexRIO_RouteTicket;




#ifdef __cplusplus
extern "C" {
#endif

/**
 * Opens a session to the FlexRIO device using a specified bitfile and creates an NiFpga session
 * reference that you can use to identify this session in subsequent FlexRIO API function calls.
 *
 * \param resourceName Specifies the resource name of the FlexRIO device.
 * \param bitfile      Specifies the FPGA image to load to the FPGA.
 * \param flags        Specifies additional NiFpga session attributes. Set this value to 0 or
 *                     bitwise OR of any of the NiFpga_OpenAttributes.
 * \param newSession   Outputs the NiFpga session reference. Destroy this session reference by
 *                     calling niFlexRIO_CloseSession when the session is no longer needed.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_OpenSession(const char* const resourceName, const char* bitfile, int32_t flags,
                      NiFpga_Session* newSession);

#define niFlexRIO_OpenDefaultBitfile(resourceName, flags, newSession)    niFlexRIO_OpenSession(resourceName, NULL, flags, newSession)

/**
 * Closes the NiFpga session, destroys the session reference, and deallocates resources associated
 * with the session. Call this function once for each unique named session that you have created.
 *
 * \param session Identifies the currently open NiFpga session.
 * \param flags   Specifies additional NiFpga session attributes. Set this value to 0 or bitwise OR
 *                of any of the NiFpga_CloseAttributes.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_CloseSession(NiFpga_Session session, int32_t flags);

/**
 * Pauses execution until the FlexRIO module I/O is ready.
 *
 * \param session     Identifies the currently open NiFpga session.
 * \param timeoutInMs Specifies a time limit in milliseconds.
 * \param ioReady     Indicates that the I/O is ready.
 * \param ioError     Outputs error codes that indicate why the I/O module is not ready. You can
 *                    retrieve a description for an error code by passing the error code to
 *                    niFlexRIO_GetError.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_WaitForIoReady(NiFpga_Session session, int32_t timeoutInMs, uint32_t* ioReady,
                         int32_t* ioError);

/**
 * Configures the Reference Clock source. You can use this function to set the onboard clock, the
 * PXI backplane clock, or an external clock as the Reference Clock.
 *
 * \param session              Identifies the currently open NiFpga session.
 * \param referenceClockSource Specifies the Reference Clock source.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureForReferenceClock(NiFpga_Session session,
                                     FlexRIO_RefClkSrc referenceClockSource);

/**
 * Configures external clock source and frequency.
 *
 * \param session     Identifies the currently open NiFpga session.
 * \param clockSource Identifies the source of the external Sample Clock.
 * \param frequency   Specifies the frequency of the external Sample Clock.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureForExternalClock(NiFpga_Session session, FlexRIO_ExtClkSrc clockSource,
                                    double frequency);

/**
 * Configures which channels are enabled for a specified stream.
 *
 * \param session         Identifies the currently open NiFpga session.
 * \param streamInstance  Identifies an instance of a stream.
 * \param channelsEnabled Specifies the channels to enable for the specified stream.
 * \param channelCount    Indicates the total number of channels parsed during execution of this
 *                        function.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureStreamEnabledChannels(NiFpga_Session session, int32_t streamInstance,
                                         const char* channelsEnabled, int32_t* channelCount);

/**
 * Configures a stream to operate in a finite transfer bound by a specified number of samples.
 * Streams configured to operate in a finite transfer will autonomously stop after completing the
 * requested number of samples or in response to an error state.
 *
 * \param session        Identifies the currently open NiFpga session.
 * \param streamInstance Identifies an instance of a stream.
 * \param numSamples     Specifies the maximum number of samples to transfer.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureStreamFinite(NiFpga_Session session, int32_t streamInstance,
                                uint64_t numberOfSamples);

/**
 * Commits attribute settings to the hardware.
 *
 * \param session Identifies the currently open NiFpga session.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_Commit(NiFpga_Session session);

/**
 * Clears samples from the stream.
 *
 * \param session        Identifies the currently open NiFpga session.
 * \param streamInstance Identifies an instance of a stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ClearStream(NiFpga_Session session, int32_t streamInstance);

/**
 * Activates a stream.
 *
 * \param session        Identifies the currently open NiFpga session.
 * \param streamInstance Identifies an instance of a stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_StartStream(NiFpga_Session session, int32_t streamInstance);

/**
 * Waits until a specified number of samples have been transferred.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param streamInstance     Identifies an instance of a stream.
 * \param waitForNSamples    Specifies the number of samples after the transfer of which execution
 *                           will resume.
 * \param timeoutInMs        Specifies a time limit in milliseconds.
 * \param samplesTransferred Indicates the actual number of samples transferred in the stream
 *                           including samples the host has not yet retrieved.
 * \param state              Returns the state of the stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_WaitForTransfer(NiFpga_Session session, int32_t instance, uint64_t waitForNSamples,
                          int32_t timeoutInMs, uint64_t* samplesTransferred, uint8_t* state);

/**
 * Reads data from a specified stream.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param streamInstance   Identifies an instance of a stream.
 * \param timeoutInMs      Specifies a time limit in milliseconds.
 * \param numberOfElements Specifies the number of elements to include in elementArray.
 * \param elementArray     Outputs the data read from the stream.
 * \param wfmInfo          Contains information about the waveform read from the stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ReadStreamI16(NiFpga_Session session, int32_t streamInstance, int32_t timeoutInMs,
                        size_t numberOfElements, int16_t* elementArray, FlexRIO_WfmInfo* wfmInfo);

/**
 * Reads data from a specified stream.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param streamInstance   Identifies an instance of a stream.
 * \param timeoutInMs      Specifies a time limit in milliseconds.
 * \param numberOfElements Specifies the number of elements to include in elementArray.
 * \param elementArray     Outputs the data read from the stream.
 * \param wfmInfo          Contains information about the waveform read from the stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ReadStreamF64(NiFpga_Session session, int32_t streamInstance, int32_t timeoutInMs,
                        size_t numberOfElements, double* elementArray, FlexRIO_WfmInfo* wfmInfo);

/**
 * Reads data from a specified stream.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param streamInstance   Identifies an instance of a stream.
 * \param timeoutInMs      Specifies a time limit in milliseconds.
 * \param numberOfWfms     Specifies the number of waveforms to output in elementArray.
 * \param numberOfElements Specifies the number of elements to include in elementArray.
 * \param elementArray     Outputs the data read from the stream.
 * \param wfmInfoArray     Contains information about waveforms read from the stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ReadStream2DI16(NiFpga_Session session, int32_t streamInstance, int32_t timeoutInMs,
                          size_t numberOfWfms, size_t numberOfElements, int16_t* elementArray, FlexRIO_WfmInfo* wfmInfoArray);

/**
 * Reads data from a specified stream.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param streamInstance   Identifies an instance of a stream.
 * \param timeoutInMs      Specifies a time limit in milliseconds.
 * \param numberOfWfms     Specifies the number of waveforms to output in elementArray.
 * \param numberOfElements Specifies the number of elements to include in elementArray.
 * \param elementArray     Outputs the data read from the stream.
 * \param wfmInfoArray     Contains information about waveforms read from the stream.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ReadStream2DF64(NiFpga_Session session, int32_t streamInstance, int32_t timeoutInMs,
                          size_t numberOfWfms, size_t numberOfElements, double* elementArray, FlexRIO_WfmInfo* wfmInfoArray);

/**
 * Looks up a control or indicator on the VI diagram in the current session's bitfile and outputs
 * the offset of the register.
 *
 * \param session Identifies the currently open NiFpga session.
 * \param name    Specifies the name of control or indicator on the VI diagram to look up in the
 *                current session's bitfile.
 * \param type    Specifies the type of the queried item. Currently only kFlexRIO_AnyType is
 *                supported.
 * \param offset  Outputs the offset of the requested register. This can be passed back to NiFpga
 *                entry points for setting and getting values from controls and indicators.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QueryBitfileRegister(NiFpga_Session session, const char* name, uint32_t type,
                               uint32_t* offset);

/**
 * Looks up a FIFO on the VI diagram in the current session's bitfile and outputs the offset of the
 * register.
 *
 * \param session Identifies the currently open NiFpga session.
 * \param name    Specifies the name of the FIFO on the VI diagram to look up in the current
 *                session's bitfile.
 * \param type    Specifies the type of the queried item. Currently only kFlexRIO_AnyType is
 *                supported.
 * \param offset  Outputs the offset of the requested register. This can be passed back to NiFpga
 *                entry points for interacting with FPGA FIFOs.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QueryBitfileFifo(NiFpga_Session session, const char* name, uint32_t type,
                           uint32_t* offset);

/**
 * Sets the value of a string attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Specifies the value to which to set the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_SetAttributeString(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeString attrID, const char* value);

/**
 * Queries the value of a string attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param bufferSize         Specifies the string buffer size.
 * \param value              Indicates the value of the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetAttributeString(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeString attrID, size_t bufferSize, char* value);

/**
 * Queries the value of a 32-bit integer attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Indicates the value of the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetAttributeInt32(NiFpga_Session session, const char* repeatedCapability,
                            FlexRIO_AttributeInt32 attrID, int32_t* value);

/**
 * Sets the value of a 32-bit integer attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Specifies the value to which to set the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_SetAttributeInt32(NiFpga_Session session, const char* repeatedCapability,
                            FlexRIO_AttributeInt32 attrID, int32_t value);

/**
 * Queries the value of a 64-bit unsigned integer attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Indicates the value of the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetAttributeUInt64(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeUInt64 attrID, uint64_t* value);

/**
 * Sets the value of a 64-bit unsigned integer attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Specifies the value to which to set the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_SetAttributeUInt64(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeUInt64 attrID, uint64_t value);

/**
 * Queries the value of a double attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Indicates the value of the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetAttributeDouble(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeDouble attrID, double* value);

/**
 * Sets the value of a double attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Specifies the value to which to set the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_SetAttributeDouble(NiFpga_Session session, const char* repeatedCapability,
                             FlexRIO_AttributeDouble attrID, double value);

/**
 * Queries the value of a Boolean attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Indicates the value of the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetAttributeBoolean(NiFpga_Session session, const char* repeatedCapability,
                              FlexRIO_AttributeBool attrID, uint32_t* value);

/**
 * Sets the value of a Boolean attribute.
 *
 * \param session            Identifies the currently open NiFpga session.
 * \param repeatedCapability Specifies a repeated capability. If attrID is based on a repeated
 *                           capability, pass a repeated capability identifier. If attrID is not
 *                           based on a repeated capability, pass NULL or an empty string.
 * \param attrID             Specifies an attribute.
 * \param value              Specifies the value to which to set the attribute.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_SetAttributeBoolean(NiFpga_Session session, const char* repeatedCapability,
                              FlexRIO_AttributeBool attrID, uint32_t value);

/**
 * Routes a signal from a specified source to a specified destination and creates a route ticket
 * that identifies the route.
 *
 * \param session     Identifies the currently open NiFpga session.
 * \param source      Specifies the source of the signal to route.
 * \param destination Specifies the destination to which to route the signal.
 * \param routeTicket Identifies a signal route.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_RouteSignal(NiFpga_Session session, const char* source, const char* destination,
                      FlexRIO_RouteTicket* routeTicket);

/**
 * Unroutes a signal specified by a route ticket.
 *
 * \param session     Identifies the currently open NiFpga session.
 * \param routeTicket Identifies a signal route. The route ticket originates from
 *                    niFlexRIO_RouteSignal.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_UnrouteSignal(NiFpga_Session session, FlexRIO_RouteTicket routeTicket);

/**
 * Configures the Start trigger for digital edge triggering.
 *
 * \param session       Identifies the currently open NiFpga session.
 * \param instance      Specifies an instance of a trigger.
 * \param triggerSource Specifies the trigger source.
 * \param sensitivity   Specifies digital trigger conditions.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureDigitalStartTrigger(NiFpga_Session session, uint32_t instance,
                                       const char* triggerSource,
                                       FlexRIO_TriggerSensitivity sensitivity);

/**
 * Configures the device to trigger itself immediately and independently of external signals.
 *
 * \param session  Identifies the currently open NiFpga session.
 * \param instance Specifies an instance of a trigger.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureImmediateTrigger(NiFpga_Session session, uint32_t instance);

/**
 * Configures the Start trigger for analog edge triggering.
 *
 * \param session    Identifies the currently open NiFpga session.
 * \param instance   Specifies an instance of a trigger.
 * \param channel    Specifies the channel to monitor for trigger conditions.
 * \param rising     Specifies whether to trigger on a rising edge. If this value is false, the
 *                   device will trigger on a falling edge.
 * \param level      Specifies the trigger level in volts.
 * \param hysteresis Specifies the size of the hysteresis window on either side of level in volts.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ConfigureAnalogEdgeTrigger(NiFpga_Session session, uint32_t instance, const char* channel,
                                     uint32_t rising, double level, double hysteresis);

/**
 * Writes calibration data to the FlexRIO device's on-board flash memory. You can find the available
 * calibration storage by retrieving the value of kFlexRIO_CalibrationStorageSize.
 *
 * \param session Identifies the currently open NiFpga session.
 * \param offset  Specifies the byte address in the device's flash memory at which to begin the
 *                operation.
 * \param data    Specifies the data to write to the device's flash memory.
 * \param size    Defines the number of bytes to read from the device's flash memory.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_WriteCalibrationData(NiFpga_Session session, uint32_t offset, const uint8_t *data,
                               size_t size);

/**
 * Reads calibration data from the FlexRIO device's on-board flash memory. You can find the
 * available calibration storage by retrieving the value of kFlexRIO_CalibrationStorageSize.
 *
 * \param session Identifies the currently open NiFpga session.
 * \param offset  Specifies the byte address in the device's flash memory at which to begin the
 *                operation.
 * \param data    Returns the data read from the device's flash memory.
 * \param size    Defines the number of bytes to read from the device's flash memory.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ReadCalibrationData(NiFpga_Session session, uint32_t offset, uint8_t* data, size_t size);

/**
 * Provides descriptions of errors.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param errorIn          Specifies the error code returned by a FlexRIO API function.
 * \param errorElaboration Outputs the error description.
 * \param size             Specifies the size of the buffer for errorElaboration.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetError(NiFpga_Session session, FlexRIO_Status errorIn, size_t size, char* errorElaboration);

/**
 * Updates verification time and verification temperature fields in the device's flash memory to
 * current system time and device temperature, respectively.
 *
 * \param session Identifies the currently open NiFpga session.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_UpdateVerificationInformation(NiFpga_Session session);

/**
 * Outputs a session reference that you can use to identify this session in calls to the NI-TClk
 * API.
 *
 * \param session          Identifies the currently open NiFpga session.
 * \param sessionReference Identifies this session. Use this session reference in calls to the NI-
 *                         TClk API.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetSessionReference(NiFpga_Session session, ViSession* sessionReference);

/**
 * Asserts DSP Reset on the same rising edge of TClk across all sessions.
 *
 * \param sessions         Specifies an array of sessions to reset.
 * \param numberOfSessions Specifies the number of sessions in the sessions array.
 * \param instance         Specifies the DSP Reset instance to reset.
 * \param size             Specifies the size of the buffer for errorElaboration.
 * \param errorElaboration Outputs the error description.
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_ResetDspSynchronously(NiFpga_Session* sessions, size_t numberOfSessions,
                                int32_t instance, size_t size, char* errorElaboration);

/**
 * TODO: add documentation for niFlexRIO_UpdateFirmware
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_UpdateFirmware(const char *resource, const char * modelName, size_t size, const uint8_t *firmware, uint8_t alwaysOverwrite);

/**
 * TODO: add documentation for niFlexRIO_GetFirmwareUpdateProgress
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_GetFirmwareUpdateProgress(const char *resource, uint32_t *progress, uint8_t *isUpdateInProgress, uint8_t *updateFailed);

/**

 *
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QSFPWrite(NiFpga_Session session, uint8_t port, uint8_t address, uint8_t data);

/**

 *
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QSFPRead(NiFpga_Session session, uint8_t port, uint8_t address, uint8_t* data);

/**

 *
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QSFPResetPorts(NiFpga_Session session);

/**

 *
 * \returns FlexRIO_Status
 */
FlexRIO_Status
_NIFLEXRIOAPI_EXPORT
niFlexRIO_QSFPEnableLowPowerMode(NiFpga_Session session, uint32_t enable);

#ifdef __cplusplus
}
#endif

#endif /* __niflexrioapi_h__ */
